@@ -1,9 +1,13 @@
 import datetime
+from functools import wraps
 
+from django.conf import settings
+from django.core.urlresolvers import reverse
 from django.http import HttpRequest
 from django.contrib.auth import SESSION_KEY
 from django.contrib.auth import authenticate
 from django.contrib.auth import login
+from django.contrib.auth.forms import UserCreationForm
 from django.contrib.auth.models import AnonymousUser
 from django.contrib.auth.models import User
 from django.contrib.sessions.middleware import SessionMiddleware
@@ -16,6 +20,28 @@ from lazysignup.decorators import allow_lazy_user
 from lazysignup.middleware import LazySignupMiddleware
 from lazysignup.management.commands import remove_expired_users
 
+from lazysignup.middleware import username_from_session
+
+class GoodUserCreationForm(UserCreationForm):
+    """ Hardcoded credentials to demonstrate that the get_credentials method
+    is being used
+    """
+    def get_credentials(self):
+        return {
+            'username': 'demo',
+            'password': 'demo',
+        }
+
+    def save(self, commit=True):
+        instance = super(GoodUserCreationForm, self).save(commit=False)
+        creds = self.get_credentials()
+        instance.username = creds['username']
+        instance.set_password(creds['password'])
+        if commit:
+            instance.save()
+        return instance
+
+
 def view(request):
     from django.http import HttpResponse
     r = HttpResponse()
@@ -26,11 +52,26 @@ def view(request):
 def lazy_view(request):
     from django.http import HttpResponse
     r = HttpResponse()
-    if request.user.has_usable_password() or request.user.is_anonymous():
+    if request.user.is_anonymous() or  request.user.has_usable_password():
         r.status_code = 500
     return r
 lazy_view = allow_lazy_user(lazy_view)
 
+def no_lazysignup(func):
+    def wrapped(*args, **kwargs):
+        old = settings.MIDDLEWARE_CLASSES
+        settings.MIDDLEWARE_CLASSES = tuple(
+                [m for m in settings.MIDDLEWARE_CLASSES 
+                if m != 'lazysignup.middleware.LazySignupMiddleware']
+            )
+        try:
+            result = func(*args, **kwargs)
+        finally:
+            settings.MIDDLEWARE_CLASSES = old
+        return result
+    return wraps(func)(wrapped)
+        
+
 class LazyTestCase(TestCase):
 
     urls = 'lazysignup.test_urls'
@@ -61,7 +102,7 @@ class LazyTestCase(TestCase):
         mock_resolve.return_value = (f, None, None)
         
         self.m.process_request(self.request)
-        self.assertEqual(self.request.session.session_key[:30], self.request.user.username)
+        self.assertEqual(self.m.get_username(self.request.session.session_key), self.request.user.username)
         self.assertEqual(False, self.request.user.has_usable_password())
         
     @mock.patch('django.core.urlresolvers.RegexURLResolver.resolve')
@@ -71,7 +112,7 @@ class LazyTestCase(TestCase):
         f = allow_lazy_user(lambda: 1)
         mock_resolve.return_value = (f, None, None)
         self.m.process_request(self.request)
-        self.assertEqual(self.request.session.session_key[:30], self.request.user.username)
+        self.assertEqual(self.m.get_username(self.request.session.session_key), self.request.user.username)
         self.assertEqual(False, self.request.user.has_usable_password())
         
     @mock.patch('django.core.urlresolvers.RegexURLResolver.resolve')
@@ -99,9 +140,9 @@ class LazyTestCase(TestCase):
         self.assertEqual(200, response.status_code)
         self.assertEqual(1, len(User.objects.all()))
         
-    def testRemoteExpiredUsers(self):
+    def testRemoveExpiredUsers(self):
         # Users wihout usable passwords who don't have a current session record should be removed.
-        u1 = User.objects.create_user('dummy', '')
+        u1 = User.objects.create_user(username_from_session('dummy'), '')
         u2 = User.objects.create_user('dummy2', '')
         s = Session(
             session_key='dummy',
@@ -223,6 +264,14 @@ class LazyTestCase(TestCase):
         response = self.client.get('/convert/')
         self.assertEqual(200, response.status_code)
         
+    @no_lazysignup
+    def testConvertAnon(self):
+        # If the Convert view gets an anonymous user, it should redirect
+        # to the login page. Not much else it can do!
+        response = self.client.get('/convert/')
+        self.assertEqual(302, response.status_code)
+        self.assertEqual('http://testserver' + settings.LOGIN_URL, response['location'])
+        
     def testConversionKeepsSameUser(self):
         self.client.get('/lazy/')
         response = self.client.post('/convert/', {
@@ -231,4 +280,43 @@ class LazyTestCase(TestCase):
             'password2': 'password',
         })
         self.assertEqual(1, len(User.objects.all()))
+        
+    @no_lazysignup
+    def testNoLazysignupDecorator(self):
+        response = self.client.get('/lazy/')
+        self.assertEqual(500, response.status_code)
+        
+    def testBadCustomConvertForm(self):
+        # Passing a form class to the conversion view that doesn't have
+        # a get_credentials method should raise an AttributeError
+        self.assertRaises(AttributeError, self.client.post, reverse('test_bad_convert'), {
+            'username': 'demo',
+            'password1': 'password',
+            'password2': 'password',
+        })
+        
+    def testGoodCustomConvertForm(self):
+        self.client.get('/lazy/')
+        response = self.client.post(reverse('test_good_convert'), {
+            'username': 'foo',
+            'password1': 'password',
+            'password2': 'password',
+        })
+        users = User.objects.all()
+        self.assertEqual(1, len(users))
+        
+        # The credentials returned by get_credentials should have been used
+        self.assertEqual(users[0], authenticate(username='demo', password='demo'))
+        
+    def testUsernameNotBasedOnSessionKey(self):
+        # The generated username should not look like the session key. While doing
+        # so isn't a security problem in itself, any client software that blindly
+        # displays the logged-in user's username risks showing most of the session
+        # key to the world.
+        fake_session_key = 'a' * 32
+        username = self.m.get_username(fake_session_key)
+        self.failIf(fake_session_key.startswith(username))
+        
+        
+        
     
\ No newline at end of file
